"""
В этом модуле представлены реализации декораций - классов, отвечающих за параметры вывода построенных префиксов
(параметр decorations у pm4py.view_petri_net). Декорации конструируются во время построения префикса.
Все встроенные классы декораций (кроме Idle) устроены по принципу декораторов, таким образом декорации можно
комбинировать.

В модуле представлены следующие декорации:
1. idle - пустые декорации, используется как основание для остальных декораций (в цепочке декораторов)
2. labels - декорации, позволяющие настроить надписи на элементах префикса
3. colors - декорации, позволяющие настроить цвет элементов префикса
4. abstract - интерфейс декораций для новых реализаций
"""

from .abstract import Decorations
from .colors import ColorsDecorations
from .idle import IdleDecorations
from .labels import LabelsDecorations

__all__ = [
    "Decorations",
    "IdleDecorations",
    "LabelsDecorations",
    "ColorsDecorations"
]
