from .abstract import OrderSettings


class IdleOrderSettings(OrderSettings):
    """Настройки порядка, при которой все конфигурации считаются несравнимыми. Таким образом, очередь с приоритетом pe
     будет работать как обычный список, т.е. возможные расширения будут рассматриваться в произвольном порядке.
     Такой порядок не является адекватным - он используется для построения развертки "до бесконечности", т.е. пока
     развертка не кончится или не будет достигнут предел количества событий в префиксе.
     """

    def config(self, e):
        return None

    def cmp_events(self, *args, **kwargs):
        return 0
